require File.expand_path(File.dirname(__FILE__) + '/helper')

describe "Dependencies" do
  describe 'when we require a dependency that have another dependency' do
    before do
      @log_level = Tennpipes::Logger::Config[:test]
      @io = StringIO.new
      Tennpipes::Logger::Config[:test] = { :log_level => :error, :stream => @io }
      Tennpipes::Logger.setup!
    end

    after do
      Tennpipes::Logger::Config[:test] = @log_level
      Tennpipes::Logger.setup!
    end

    it 'should raise an error without reloading it twice' do
      capture_io do
        assert_raises(RuntimeError) do
          Tennpipes.require_dependencies(
            Tennpipes.root("fixtures/dependencies/a.rb"),
            Tennpipes.root("fixtures/dependencies/b.rb"),
            Tennpipes.root("fixtures/dependencies/c.rb"),
            Tennpipes.root("fixtures/dependencies/d.rb")
          )
        end
      end
      assert_equal 1, D
      assert_match /RuntimeError - SomeThing/, @io.string
    end

    it 'should resolve dependency problems' do
      capture_io do
        Tennpipes.require_dependencies(
          Tennpipes.root("fixtures/dependencies/a.rb"),
          Tennpipes.root("fixtures/dependencies/b.rb"),
          Tennpipes.root("fixtures/dependencies/c.rb")
        )
      end
      assert_equal ["B", "C"], A_result
      assert_equal "C", B_result
      assert_equal "", @io.string
    end

    it 'should remove partially loaded constants' do
      capture_io do
        Tennpipes.require_dependencies(
          Tennpipes.root("fixtures/dependencies/circular/e.rb"),
          Tennpipes.root("fixtures/dependencies/circular/f.rb"),
          Tennpipes.root("fixtures/dependencies/circular/g.rb")
        )
      end
      assert_equal ["name"], F.fields
      assert_equal "", @io.string
    end
  end
end
